<?php

namespace Drupal\course_attendance\Form;

use Drupal\Core\Access\AccessResultAllowed;
use Drupal\Core\Access\AccessResultForbidden;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Flood\FloodInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\course_attendance\Plugin\course\CourseObject\CourseObjectAttendance;

class CourseAttendanceCodeForm extends FormBase {

  /**
   * The flood service.
   *
   * @var \Drupal\Core\Flood\FloodInterface
   */
  protected $flood;

  /**
   * FloodProtectedForm constructor.
   *
   * @param \Drupal\Core\Flood\FloodInterface $flood
   *   The flood service.
   */
  public function __construct(FloodInterface $flood) {
    $this->flood = $flood;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('flood')
    );
  }

  public function getFormId() {
    return 'course_signup_confirmation_form';
  }

  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['code'] = [
      '#title' => t('Code'),
      '#type' => 'textfield',
      '#maxlength' => 16,
      '#size' => 16,
      '#description' => t('You must enter an attendance code to continue.'),
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => t('Submit'),
    ];

    return $form;
  }

  public function validateForm(array &$form, FormStateInterface $form_state) {
    if (!$this->flood->isAllowed('course_signup_code', 10, 60, \Drupal::currentUser()->id())) {
      $form_state->setError($form['code'], t('Sorry, you are doing that too many times. Please wait 1 minute and try again.'));
      return;
    }
    $this->flood->register('course_signup_code', 60, \Drupal::currentUser()->id());

    if (!$courseObject = CourseObjectAttendance::findObjectByCode($form_state->getValue('code'))) {
      $form_state->setError($form['code'], t('The code you entered is not valid.'));
      $form['code']['#value'] = '';
      return;
    }

    /* @var \Drupal\Core\Access\AccessResultInterface $result */
    $result = $courseObject->access('attendance', \Drupal::currentUser(), TRUE);

    if ($result->isForbidden()) {
      $tokens = [];
      $tokens['course'] = $courseObject->getCourse();
      $message_config = \Drupal::config('course_sms.messages');
      $message = \Drupal::token()->replace($message_config->get($result->getReason()), $tokens, ['sanitize' => FALSE, 'clear' => TRUE]);

      $form_state->setError($form['code'], $message);
      return;
    }
  }

  public function submitForm(array &$form, FormStateInterface $form_state) {
    $account = \Drupal::currentUser();
    /* @var \Drupal\course_attendance\Plugin\course\CourseObject\CourseObjectAttendance $courseObject */
    $courseObject = CourseObjectAttendance::findObjectByCode($form_state->getValue('code'));


    $courseObject->markAttended($account);

    $tokens = [];
    $tokens['course'] = $courseObject->getCourse();
    $tokens['user'] = $account;
    $message_config = \Drupal::config('course_sms.messages');
    $message = \Drupal::token()->replace($message_config->get('confirmation_registered'), $tokens, ['sanitize' => FALSE, 'clear' => TRUE]);
    \Drupal::messenger()->addMessage($message);
  }

  /**
   * Route access callback.
   */
  function codeFormAccess() {
    $config = \Drupal::config('course_sms.settings');
    if (\Drupal::currentUser()->isAuthenticated() && $config->get('code_form_enabled')) {
      return AccessResultAllowed::allowed();
    }
    else {
      return AccessResultForbidden::forbidden();
    }
  }

}
