<?php

namespace Drupal\gcourse\Plugin\Group\Relation;

use Drupal;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\course\Entity\CourseType;
use Drupal\group\Entity\GroupInterface;
use \Drupal\group\Plugin\Group\Relation\GroupRelationBase;
use Drupal\group\Plugin\GroupContentEnablerBase;

/**
 * Provides a content enabler for courses.
 *
 * @GroupRelationType(
 *   id = "course",
 *   label = @Translation("Course"),
 *   description = @Translation("Adds courses to groups."),
 *   entity_type_id = "course",
 *   entity_access = TRUE,
 *   pretty_path_key = "course",
 *   reference_label = @Translation("Title"),
 *   reference_description = @Translation("The name of the course to add to the group"),
 *   deriver = "Drupal\gcourse\Plugin\Group\Relation\CourseEntityDeriver",
 *   handlers = {
 *     "access" = "Drupal\group\Plugin\GroupContentAccessControlHandler",
 *     "permission_provider" = "Drupal\group\Plugin\GroupContentPermissionProvider",
 *   }
 * )
 */
class CourseEntity extends GroupRelationBase {

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    $config = parent::defaultConfiguration();
    $config['entity_cardinality'] = 1;
    return $config;
  }

  /**
   * {@inheritdoc}
   */
  public function getGroupOperations(GroupInterface $group) {
    $account = Drupal::currentUser();
    $plugin_id = $this->getPluginId();
    $type = $this->getEntityBundle();
    $operations = [];

    if ($group->hasPermission("create $plugin_id entity", $account)) {
      $route_params = ['group' => $group->id(), 'plugin_id' => $plugin_id];
      $bundle = CourseType::load($type);
      $operations["course-create-$type"] = [
        'title' => $this->t('Add @type', ['@type' => $bundle->label()]),
        'url' => new Url('entity.group_relationship.create_form', $route_params),
        'weight' => 30,
      ];
    }

    return $operations;
  }

  /**
   * {@inheritdoc}
   */
  public function calculateDependencies() {
    $dependencies = parent::calculateDependencies();
    $dependencies['config'][] = 'course.type.' . $this->getRelationType()->getEntityBundle();
    return $dependencies;
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    // Disable the entity cardinality field as the functionality of this module
    // relies on a cardinality of 1. We don't just hide it, though, to keep a UI
    // that's consistent with other group relations.
    $info = $this->t("This field has been disabled by the plugin to guarantee the functionality that's expected of it.");
    $form['entity_cardinality']['#disabled'] = TRUE;
    $form['entity_cardinality']['#description'] .= '<br /><em>' . $info . '</em>';

    return $form;
  }

}
