<?php

namespace Drupal\course\Plugin\Action;

use Drupal;
use Drupal\Core\Entity\EntityCreateAccessCheck;
use Drupal\Core\Render\Markup;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\course\Entity\Course;
use \Drupal\views_bulk_operations\Action\ViewsBulkOperationsActionBase;

/**
 * Action description.
 *
 * @Action(
 *   id = "course_add_enrollment_action",
 *   label = @Translation("Enroll user"),
 *   type = "user"
 * )
 */
class CourseEnrollmentCreateAction extends ViewsBulkOperationsActionBase {

  use StringTranslationTrait;

  /**
   * {@inheritdoc}
   */
  public function execute($entity = NULL) {
    /* @var $url Drupal\Core\Url */
    $url = $this->context['redirect_url'];
    $cid = $url->getRouteParameters()['course'];
    $course = Course::load($cid);
    $course->enroll($entity, ['enrollmenttype' => 'bulk']);
    $username = Markup::create($entity->label());
    $course_title = Markup::create($course->label());
    return $this->t('Enrolled @name in @course', [
      '@name' => $username,
      '@course' => $course_title,
    ]);
  }

  /**
   * If the user can edit the course or create enrollments, they can enroll
   * users.
   *
   * {@inheritdoc}
   */
  public function access($object, AccountInterface $account = NULL, $return_as_object = FALSE) {
    $course = \Drupal::entityTypeManager()->getStorage('course')->load($this->view->args[0]);

    $course_update = $course->access('update', $account);

    $enrollment_create = \Drupal::entityTypeManager()
      ->getAccessControlHandler('course_enrollment')
      ->createAccess('course_enrollment', $account);

    $access = $course_update || $enrollment_create;

    return $return_as_object ? Drupal\Core\Access\AccessResult::allowedIf($access) : $access;
  }

}
