<?php

namespace Drupal\course\Plugin\Action;

use Drupal\Core\Access\AccessResultAllowed;
use Drupal\course\Entity\CourseObject;
use Drupal\course\Entity\CourseEnrollment;
use Drupal;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use \Drupal\views_bulk_edit\Plugin\Action\ModifyEntityValues;
use \Drupal\views_bulk_operations\Action\ViewsBulkOperationsActionBase;
use function count;
use function course_get_course;

/**
 * Action description.
 *
 * @todo not yet working
 *
 * @Action(
 *   id = "course_edit_enrollment_action",
 *   label = @Translation("Edit enrollment"),
 *   type = "course_enrollment"
 * )
 */
class CourseEnrollmentEditAction extends ModifyEntityValues {

  use StringTranslationTrait;

  /**
   * {@inheritdoc}
   */
  public function execute($entity = NULL) {
    parent::execute($entity);

    $course = $entity->getCourse();
    $account = $entity->getUser();

    foreach ($this->getConfiguration()['course_objects'] as $id => $complete) {
      if ($complete !== '') {
        // There was a change

        $courseObject = CourseObject::load($id);
        $fulfillment = $courseObject->getFulfillment($account);

        if ($complete == 1) {
          // Completed
          $fulfillment->setOption('message', "Fulfillment completed via bulk action.");
          $fulfillment->setComplete();
        }

        if ($complete == -1) {
          // Delete attempt
          $fulfillment->delete();
        }

        if ($complete == 0) {
          // Fail user
          $fulfillment->setOption('message', "Fulfillment failed via bulk action.");
          $fulfillment->setComplete(FALSE);
          $fulfillment->setGrade(0);
        }

        $fulfillment->save();
      }
    }

    // Recheck requirements.
    $entity->track();

    return $this->t('Updated enrollment for @name', [
      '@name' => $account->name->value,
      '@course' => $course->label(),
    ]);

  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    // Load the first enrollment.
    $selections = $form_state->getStorage()['views_bulk_operations']['list'];
    $course_enrollment = CourseEnrollment::load(reset($selections)[0]);
    $course = $course_enrollment->getCourse();

    // Check if this action is being performed on a single user, and set the
    // account accordingly.
    $account = NULL;
    if (count($form_state->getStorage()['views_bulk_operations']['list']) === 1) {
      // Only one user and course, so let's prefill values.
      $account = $course_enrollment->getUser();
    }

    // Get course objects, with or without a single user account information.
    $objects = $course->getObjects();

    // Build a list of a single user's fulfillments.
    $fulfillments = NULL;
    if ($account) {
      $fulfillments = array();
      foreach ($objects as $courseObject) {
        $fulfillments[$courseObject->id()] = $courseObject->getFulfillment($account);
      }
    }

    $form['course_objects'] = array(
      '#title' => t('Set completion status'),
      '#description' => t('Set the status of a course object to be applied to selected users.'),
      '#type' => 'fieldset',
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
      '#tree' => TRUE,
      '#prefix' => '<span id="course-objects-wrapper">',
      '#suffix' => '</span>',
    );

    foreach ($objects as $courseObject) {
      $form['course_objects'][$courseObject->id()] = array(
        '#type' => 'select',
        '#title' => $courseObject->getTitle(),
        '#options' => array(
          '' => '- no change - ',
          1 => t('Complete'),
          -1 => t('Incomplete'),
          0 => t('Failed'),
        ),
        '#default_value' => $fulfillments ? $fulfillments[$courseObject->id()]->isComplete() : NULL,
      );
    }

    return $form;
  }

  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) :void {
    parent::submitConfigurationForm($form, $form_state);
    $this->configuration['course_objects'] = $form_state->getValue('course_objects');
  }
}
