<?php

namespace Drupal\course\Plugin\course\CourseOutline;

use Drupal\Component\Utility\Html;
use Drupal\Core\Link;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Url;
use Drupal\course\Entity\Course;
use Drupal\course\Entity\CourseObject;
use Drupal\course\Plugin\CourseOutlinePluginBase;
use function drupal_render;
use function entity_load;
use function entity_view;

/**
 * @CourseOutline(
 *   id = "course",
 *   label = @Translation("Course"),
 *   description = @Translation("Displays course objects in an HTML list."),
 * )
 */
class CourseOutlineList extends CourseOutlinePluginBase {

  /**
   * {@inheritdoc}
   */
  function render(Course $course, AccountInterface $account) {
    // Iterate over objects.
    $workflow = array();
    $img = NULL;
    foreach ($course->getObjects() as $key => $courseObject) {
      if ($courseObject->access('see', $account)) {
        // The item will be in the list only if the user can see it. If they can
        // take it, entity_view() will output a link instead of text.
        $entity = CourseObject::load($courseObject->id());
        $render_controller = \Drupal::entityTypeManager()
          ->getViewBuilder('course_object');
        $item = $render_controller->view($entity);
        $classes = [];
        $class_prefix = 'course-object-';

        // User can take this course object.
        if ($courseObject->access('take', $account)) {
          $classes[] = $class_prefix . 'accessible';
          // Step is complete.
          if ($courseObject->getFulfillment($account)->isComplete()) {
            $classes[] = $class_prefix . 'completed';
          }
          elseif ($courseObject->getFulfillment($account)->id()) {
            $classes[] = $class_prefix . 'in-progress';
          }
          if ($course->getActive() === $courseObject) {
            $classes[] = $class_prefix . 'active';
          }
        }
        else {
          $classes[] = $class_prefix . 'not-accessible';
        }

        if ($courseObject->isRequired()) {
          $classes[] = $class_prefix . 'required';
        }
        else {
          $classes[] = $class_prefix . 'optional';
        }

        $classes[] = $class_prefix . $courseObject->id();
        $classes[] = 'course-object-type-' . Html::cleanCssIdentifier($courseObject->getComponent());
        $classes[] = 'course-outline-list-item';

        $item['#wrapper_attributes']['class'] = $classes;

        // Allow other modules to modify this list item.
        $courseObject->overrideOutlineListItem($item);

        // Add this item to the list.
        $workflow[] = $item;
      }
    }

    if ($course->getEnrollment($account)->isComplete()) {
      $image = [
        '#uri' => 'core/misc/icons/73b355/check.svg',
        '#alt' => t('Completed'),
        '#theme' => 'image',
      ];
      $workflow[] = array(
        '#markup' => \Drupal::service('renderer')->render($image) . Link::fromTextAndUrl(t('Complete'), Url::fromRoute('course.complete', ['course' => $course->id()]))->toString(),
        '#wrapper_attributes' => [
          'class' => ['course-outline-list-item', 'course-object-completed'],
        ]
      );
    }

    $output = [];
    if ($workflow) {
      $page = [];
      $page['course_outline']['#theme'] = 'item_list';
      $page['course_outline']['#attributes'] = [
        'class' => [
          'course-outline-list',
        ],
      ];
      $page['course_outline']['#items'] = $workflow;
      return $page;
    }
    return $output;
  }

}
