<?php

namespace Drupal\course\View;

use Drupal;
use Drupal\Component\Utility\Html;
use Drupal\Core\Entity\EntityViewBuilder;
use Drupal\Core\Link;

class CourseObjectViewBuilder extends EntityViewBuilder {

  public function build(array $build) {
    $build = parent::build($build);

    $courseObject = $build['#course_object'];
    // When viewing, we use the current user.
    $account = Drupal::currentUser();

    $step = [];
    $step['image'] = '';
    $step['status'] = '';
    $status_text = '';
    $access_msgs = [];

    if ($courseObject->access('see', $account)) {
      if ($courseObject->access('take', $account)) {
        // User can take this course object.
        $step['link'] = $courseObject->getUrl();
        $status_text = t('Not started');

        // Step is complete.
        if ($courseObject->getFulfillment($account)->isComplete()) {
          $status_text = t('Complete');
          $step['image'] = 'core/misc/icons/73b355/check.svg';
        }
        elseif ($courseObject->getFulfillment($account)->id()) {
          $status_text = t('In progress');
          $step['image'] = '';
        }
      }
      else {
        // User cannot access this step yet.
        $status_text = t('Locked');
        if (!empty($courseObject->getAccessMessages())) {
          $access_msgs = [
            '#theme' => 'item_list',
            '#list_type' => 'ul',
            '#items' => $courseObject->getAccessMessages(),
            '#wrapper_attributes' => ['class' => 'course-object-status-messages'],
          ];

        }
      }

      if ($status_text) {
        $status = [
          '#type' => 'html_tag',
          '#tag' => 'span',
          '#value' => $status_text,
          '#attributes' => ['class' => 'course-object-status-text'],
        ];
        $step['status'] = \Drupal::service('renderer')->render($status);
        if ($access_msgs) {
          $step['status'] .= \Drupal::service('renderer')->render($access_msgs);
        }
      }

      $img = [];
      if (!empty($step['image'])) {
        $img = ['#theme' => 'image', '#uri' => $step['image'], '#alt' => strip_tags($step['status'])];
        $img['#attributes'] = [
          'class' => 'course-object-image',
        ];
      }

      $build['course_outline_item'] = [
        '#type' => 'container',
        '#attributes' => [
          'class' => [
            'course-object-wrapper',
          ],
        ],
      ];
      $build['course_outline_item']['course_outline_image'] = $img;

      $link = $courseObject->getTitle();
      if ($courseObject->access('take', $account)) {
        $link = Link::fromTextAndUrl($link, $courseObject->getUrl());
        $link = $link->toString();
      }
      $object_title = [
        '#type' => 'html_tag',
        '#tag' => 'div',
        '#value' => $link,
        '#attributes' => [
          'class' => 'course-object-title',
        ],
      ];

      $build['course_outline_item']['course_outline_link'] = $object_title;

      if (!empty($step['status'])) {
        $build['course_outline_item']['course_outline_status'] = [
          '#type' => 'html_tag',
          '#tag' => 'div',
          '#value' => $step['status'],
          '#attributes' => [
            'class' => 'course-object-status',
          ],
        ];
      }

      // No cache as this list always changes.
      $build['#cache']['max-age'] = 0;
    }

    return $build;
  }

}
